@@ -77,8 +77,6 @@ extern void lttng_ring_buffer_client_discard_exit(void);
 extern void lttng_ring_buffer_client_discard_rt_exit(void);
 extern void lttng_ring_buffer_metadata_client_exit(void);
 
-volatile enum ust_loglevel ust_loglevel;
-
 int ustctl_release_handle(int sock, int handle)
 {
 	struct ustcomm_ust_msg lum;
@@ -111,8 +109,10 @@ int ustctl_release_object(int sock, struct lttng_ust_object_data *data)
 				ret = -errno;
 				return ret;
 			}
+			data->u.channel.wakeup_fd = -1;
 		}
 		free(data->u.channel.data);
+		data->u.channel.data = NULL;
 		break;
 	case LTTNG_UST_OBJECT_TYPE_STREAM:
 		if (data->u.stream.shm_fd >= 0) {
@@ -121,6 +121,7 @@ int ustctl_release_object(int sock, struct lttng_ust_object_data *data)
 				ret = -errno;
 				return ret;
 			}
+			data->u.stream.shm_fd = -1;
 		}
 		if (data->u.stream.wakeup_fd >= 0) {
 			ret = close(data->u.stream.wakeup_fd);
@@ -128,6 +129,7 @@ int ustctl_release_object(int sock, struct lttng_ust_object_data *data)
 				ret = -errno;
 				return ret;
 			}
+			data->u.stream.wakeup_fd = -1;
 		}
 		break;
 	case LTTNG_UST_OBJECT_TYPE_EVENT:
@@ -1590,6 +1592,19 @@ void ustctl_flush_buffer(struct ustctl_consumer_stream *stream,
 		consumer_chan->chan->handle);
 }
 
+void ustctl_clear_buffer(struct ustctl_consumer_stream *stream)
+{
+	struct lttng_ust_lib_ring_buffer *buf;
+	struct ustctl_consumer_channel *consumer_chan;
+
+	assert(stream);
+	buf = stream->buf;
+	consumer_chan = stream->chan;
+	lib_ring_buffer_switch_slow(buf, SWITCH_ACTIVE,
+		consumer_chan->chan->handle);
+	lib_ring_buffer_clear_reader(buf, consumer_chan->chan->handle);
+}
+
 static
 struct lttng_ust_client_lib_ring_buffer_client_cb *get_client_cb(
 		struct lttng_ust_lib_ring_buffer *buf,
