@@ -96,8 +96,8 @@ void ust_malloc_spin_unlock(pthread_mutex_t *lock)
 #define pthread_mutex_lock ust_malloc_spin_lock
 #define pthread_mutex_unlock ust_malloc_spin_unlock
 static DEFINE_URCU_TLS(int, malloc_nesting);
-#undef ust_malloc_spin_unlock
-#undef ust_malloc_spin_lock
+#undef pthread_mutex_unlock
+#undef pthread_mutex_lock
 #undef calloc
 
 /*
