@@ -69,6 +69,10 @@ struct packet_header {
 	} ctx;
 };
 
+struct lttng_client_ctx {
+	size_t packet_context_len;
+	size_t event_context_len;
+};
 
 static inline uint64_t lib_ring_buffer_clock_read(struct channel *chan)
 {
@@ -76,15 +80,29 @@ static inline uint64_t lib_ring_buffer_clock_read(struct channel *chan)
 }
 
 static inline
-size_t ctx_get_size(size_t offset, struct lttng_ctx *ctx,
-		enum app_ctx_mode mode)
+size_t ctx_get_aligned_size(size_t offset, struct lttng_ctx *ctx,
+		size_t ctx_len)
 {
-	int i;
 	size_t orig_offset = offset;
 
 	if (caa_likely(!ctx))
 		return 0;
 	offset += lib_ring_buffer_align(offset, ctx->largest_align);
+	offset += ctx_len;
+	return offset - orig_offset;
+}
+
+static inline
+void ctx_get_struct_size(struct lttng_ctx *ctx, size_t *ctx_len,
+		enum app_ctx_mode mode)
+{
+	int i;
+	size_t offset = 0;
+
+	if (caa_likely(!ctx)) {
+		*ctx_len = 0;
+		return;
+	}
 	for (i = 0; i < ctx->nr_fields; i++) {
 		if (mode == APP_CTX_ENABLED) {
 			offset += ctx->fields[i].get_size(&ctx->fields[i], offset);
@@ -106,7 +124,7 @@ size_t ctx_get_size(size_t offset, struct lttng_ctx *ctx,
 			}
 		}
 	}
-	return offset - orig_offset;
+	*ctx_len = offset;
 }
 
 static inline
@@ -160,7 +178,8 @@ static __inline__
 size_t record_header_size(const struct lttng_ust_lib_ring_buffer_config *config,
 				 struct channel *chan, size_t offset,
 				 size_t *pre_header_padding,
-				 struct lttng_ust_lib_ring_buffer_ctx *ctx)
+				 struct lttng_ust_lib_ring_buffer_ctx *ctx,
+				 struct lttng_client_ctx *client_ctx)
 {
 	struct lttng_channel *lttng_chan = channel_get_private(chan);
 	struct lttng_event *event = ctx->priv;
@@ -205,12 +224,16 @@ size_t record_header_size(const struct lttng_ust_lib_ring_buffer_config *config,
 	}
 	if (lttng_ctx) {
 		/* 2.8+ probe ABI. */
-		offset += ctx_get_size(offset, lttng_ctx->chan_ctx, APP_CTX_ENABLED);
-		offset += ctx_get_size(offset, lttng_ctx->event_ctx, APP_CTX_ENABLED);
+		offset += ctx_get_aligned_size(offset, lttng_ctx->chan_ctx,
+				client_ctx->packet_context_len);
+		offset += ctx_get_aligned_size(offset, lttng_ctx->event_ctx,
+				client_ctx->event_context_len);
 	} else {
 		/* Pre 2.8 probe ABI. */
-		offset += ctx_get_size(offset, lttng_chan->ctx, APP_CTX_DISABLED);
-		offset += ctx_get_size(offset, event->ctx, APP_CTX_DISABLED);
+		offset += ctx_get_aligned_size(offset, lttng_chan->ctx,
+				client_ctx->packet_context_len);
+		offset += ctx_get_aligned_size(offset, event->ctx,
+				client_ctx->event_context_len);
 	}
 	*pre_header_padding = padding;
 	return offset - orig_offset;
@@ -379,10 +402,11 @@ static
 size_t client_record_header_size(const struct lttng_ust_lib_ring_buffer_config *config,
 				 struct channel *chan, size_t offset,
 				 size_t *pre_header_padding,
-				 struct lttng_ust_lib_ring_buffer_ctx *ctx)
+				 struct lttng_ust_lib_ring_buffer_ctx *ctx,
+				 void *client_ctx)
 {
 	return record_header_size(config, chan, offset,
-				  pre_header_padding, ctx);
+				  pre_header_padding, ctx, client_ctx);
 }
 
 /**
@@ -694,8 +718,27 @@ int lttng_event_reserve(struct lttng_ust_lib_ring_buffer_ctx *ctx,
 		      uint32_t event_id)
 {
 	struct lttng_channel *lttng_chan = channel_get_private(ctx->chan);
+	struct lttng_event *event = ctx->priv;
+	struct lttng_stack_ctx *lttng_ctx = ctx->priv2;
+	struct lttng_client_ctx client_ctx;
 	int ret, cpu;
 
+	/* Compute internal size of context structures. */
+
+	if (lttng_ctx) {
+		/* 2.8+ probe ABI. */
+		ctx_get_struct_size(lttng_ctx->chan_ctx, &client_ctx.packet_context_len,
+				APP_CTX_ENABLED);
+		ctx_get_struct_size(lttng_ctx->event_ctx, &client_ctx.event_context_len,
+				APP_CTX_ENABLED);
+	} else {
+		/* Pre 2.8 probe ABI. */
+		ctx_get_struct_size(lttng_chan->ctx, &client_ctx.packet_context_len,
+				APP_CTX_DISABLED);
+		ctx_get_struct_size(event->ctx, &client_ctx.event_context_len,
+				APP_CTX_DISABLED);
+	}
+
 	cpu = lib_ring_buffer_get_cpu(&client_config);
 	if (cpu < 0)
 		return -EPERM;
@@ -714,7 +757,7 @@ int lttng_event_reserve(struct lttng_ust_lib_ring_buffer_ctx *ctx,
 		WARN_ON_ONCE(1);
 	}
 
-	ret = lib_ring_buffer_reserve(&client_config, ctx);
+	ret = lib_ring_buffer_reserve(&client_config, ctx, &client_ctx);
 	if (caa_unlikely(ret))
 		goto put;
 	if (caa_likely(ctx->ctx_len
