@@ -26,7 +26,7 @@
 #include <stdio.h>
 
 #include <urcu/arch.h>
-#include <urcu-bp.h>
+#include <urcu/urcu-bp.h>
 #include <urcu/hlist.h>
 #include <urcu/uatomic.h>
 #include <urcu/compiler.h>
@@ -166,7 +166,7 @@ static void release_probes(void *old)
 	if (old) {
 		struct tp_probes *tp_probes = caa_container_of(old,
 			struct tp_probes, probes[0]);
-		synchronize_rcu();
+		urcu_bp_synchronize_rcu();
 		free(tp_probes);
 	}
 }
@@ -750,7 +750,7 @@ void __tracepoint_probe_prune_release_queue(void)
 	release_queue_need_update = 0;
 
 	/* Wait for grace period between all sync_callsites and free. */
-	synchronize_rcu();
+	urcu_bp_synchronize_rcu();
 
 	cds_list_for_each_entry_safe(pos, next, &release_probes, u.list) {
 		cds_list_del(&pos->u.list);
@@ -841,7 +841,7 @@ void tracepoint_probe_update_all(void)
 
 	tracepoint_update_probes();
 	/* Wait for grace period between update_probes and free. */
-	synchronize_rcu();
+	urcu_bp_synchronize_rcu();
 	cds_list_for_each_entry_safe(pos, next, &release_probes, u.list) {
 		cds_list_del(&pos->u.list);
 		free(pos);
@@ -987,17 +987,17 @@ void exit_tracepoint(void)
 
 void tp_rcu_read_lock_bp(void)
 {
-	rcu_read_lock_bp();
+	urcu_bp_read_lock();
 }
 
 void tp_rcu_read_unlock_bp(void)
 {
-	rcu_read_unlock_bp();
+	urcu_bp_read_unlock();
 }
 
 void *tp_rcu_dereference_sym_bp(void *p)
 {
-	return rcu_dereference_bp(p);
+	return urcu_bp_dereference(p);
 }
 
 /*
